"use strict";
var __assign = (this && this.__assign) || Object.assign || function(t) {
    for (var s, i = 1, n = arguments.length; i < n; i++) {
        s = arguments[i];
        for (var p in s) if (Object.prototype.hasOwnProperty.call(s, p))
            t[p] = s[p];
    }
    return t;
};
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
var react_1 = __importDefault(require("react"));
var Matcher = (function () {
    function Matcher(name, options, factory) {
        if (options === void 0) { options = {}; }
        if (factory === void 0) { factory = null; }
        if (process.env.NODE_ENV !== 'production') {
            if (!name || name.toLowerCase() === 'html') {
                throw new Error("The matcher name \"" + name + "\" is not allowed.");
            }
        }
        this.options = __assign({}, options);
        this.propName = name;
        this.inverseName = "no" + (name.charAt(0).toUpperCase() + name.slice(1));
        this.factory = factory;
    }
    Matcher.prototype.createElement = function (match, props) {
        if (props === void 0) { props = {}; }
        var element = null;
        if (typeof this.factory === 'function') {
            element = this.factory(match, props);
        }
        else {
            element = this.replaceWith(match, props);
        }
        if (process.env.NODE_ENV !== 'production') {
            if (typeof element !== 'string' && !react_1.default.isValidElement(element)) {
                throw new Error("Invalid React element created from " + this.constructor.name + ".");
            }
        }
        return element;
    };
    Matcher.prototype.replaceWith = function (match, props) {
        if (props === void 0) { props = {}; }
        if (process.env.NODE_ENV !== 'production') {
            throw new Error(this.constructor.name + " must return a React element.");
        }
        return match;
    };
    Matcher.prototype.asTag = function () {
        if (process.env.NODE_ENV !== 'production') {
            throw new Error(this.constructor.name + " must define the HTML tag name it will render.");
        }
        return '';
    };
    Matcher.prototype.match = function (string) {
        if (process.env.NODE_ENV !== 'production') {
            throw new Error(this.constructor.name + " must define a pattern matcher.");
        }
        return null;
    };
    Matcher.prototype.doMatch = function (string, pattern, callback) {
        var matches = string.match(pattern instanceof RegExp ? pattern : new RegExp(pattern, 'i'));
        if (!matches) {
            return null;
        }
        return __assign({}, callback(matches), { match: matches[0] });
    };
    Matcher.prototype.onBeforeParse = function (content, props) {
        return content;
    };
    Matcher.prototype.onAfterParse = function (content, props) {
        return content;
    };
    return Matcher;
}());
exports.default = Matcher;
