"use strict";
var __assign = (this && this.__assign) || Object.assign || function(t) {
    for (var s, i = 1, n = arguments.length; i < n; i++) {
        s = arguments[i];
        for (var p in s) if (Object.prototype.hasOwnProperty.call(s, p))
            t[p] = s[p];
    }
    return t;
};
var __rest = (this && this.__rest) || function (s, e) {
    var t = {};
    for (var p in s) if (Object.prototype.hasOwnProperty.call(s, p) && e.indexOf(p) < 0)
        t[p] = s[p];
    if (s != null && typeof Object.getOwnPropertySymbols === "function")
        for (var i = 0, p = Object.getOwnPropertySymbols(s); i < p.length; i++) if (e.indexOf(p[i]) < 0)
            t[p[i]] = s[p[i]];
    return t;
};
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
var react_1 = __importDefault(require("react"));
var Element_1 = __importDefault(require("./Element"));
var constants_1 = require("./constants");
var ELEMENT_NODE = 1;
var TEXT_NODE = 3;
var INVALID_ROOTS = ['!DOC', 'HTML', 'HEAD', 'BODY'];
var ROOT_COMPARE_LENGTH = 4;
var ARIA_COMPARE_LENGTH = 5;
var Parser = (function () {
    function Parser(markup, props, matchers, filters) {
        if (props === void 0) { props = {}; }
        if (matchers === void 0) { matchers = []; }
        if (filters === void 0) { filters = []; }
        this.content = [];
        if (process.env.NODE_ENV !== 'production') {
            if (markup && typeof markup !== 'string') {
                throw new TypeError('Interweave parser requires a valid string.');
            }
        }
        this.props = props;
        this.matchers = matchers;
        this.filters = filters;
        this.keyIndex = -1;
        this.doc = this.createDocument(markup || '');
    }
    Parser.prototype.applyAttributeFilters = function (name, value) {
        return this.filters.reduce(function (nextValue, filter) { return filter.attribute(name, nextValue); }, value);
    };
    Parser.prototype.applyNodeFilters = function (name, node) {
        return this.filters.reduce(function (nextNode, filter) {
            return nextNode && typeof filter.node === 'function' ? filter.node(name, nextNode) : nextNode;
        }, node);
    };
    Parser.prototype.applyMatchers = function (string, parentConfig) {
        var _this = this;
        var elements = [];
        var props = this.props;
        var matchedString = string;
        var parts = null;
        this.matchers.forEach(function (matcher) {
            var tagName = matcher.asTag().toLowerCase();
            var config = _this.getTagConfig(tagName);
            if (props[matcher.inverseName] ||
                constants_1.TAGS_BLACKLIST[tagName] ||
                (!props.disableWhitelist && !constants_1.TAGS[tagName])) {
                return;
            }
            if (!_this.canRenderChild(parentConfig, config)) {
                return;
            }
            while ((parts = matcher.match(matchedString))) {
                var _a = parts, match = _a.match, partProps = __rest(_a, ["match"]);
                matchedString = matchedString.replace(match, "#{{" + elements.length + "}}#");
                _this.keyIndex += 1;
                elements.push(matcher.createElement(match, __assign({}, props, partProps, { key: _this.keyIndex })));
            }
        });
        if (elements.length === 0) {
            return matchedString;
        }
        var matchedArray = [];
        var lastIndex = 0;
        while ((parts = matchedString.match(/#\{\{(\d+)\}\}#/))) {
            var _a = parts, no = _a[1];
            var _b = parts.index, index = _b === void 0 ? 0 : _b;
            if (lastIndex !== index) {
                matchedArray.push(matchedString.slice(lastIndex, index));
            }
            matchedArray.push(elements[parseInt(no, 10)]);
            lastIndex = index + parts[0].length;
            matchedString = matchedString.replace("#{{" + no + "}}#", "%{{" + no + "}}%");
        }
        if (lastIndex < matchedString.length) {
            matchedArray.push(matchedString.slice(lastIndex));
        }
        return matchedArray;
    };
    Parser.prototype.canRenderChild = function (parentConfig, childConfig) {
        if (!parentConfig.tagName || !childConfig.tagName) {
            return false;
        }
        if (parentConfig.children &&
            parentConfig.children.length > 0 &&
            parentConfig.children.indexOf(childConfig.tagName) === -1) {
            return false;
        }
        if (childConfig.parent &&
            childConfig.parent.length > 0 &&
            childConfig.parent.indexOf(parentConfig.tagName) === -1) {
            return false;
        }
        if (!parentConfig.self && parentConfig.tagName === childConfig.tagName) {
            return false;
        }
        if (!parentConfig.block && childConfig.type === constants_1.TYPE_BLOCK) {
            return false;
        }
        if (!parentConfig.inline && childConfig.type === constants_1.TYPE_INLINE) {
            return false;
        }
        return true;
    };
    Parser.prototype.convertLineBreaks = function (markup) {
        var _a = this.props, noHtml = _a.noHtml, noHtmlExceptMatchers = _a.noHtmlExceptMatchers, disableLineBreaks = _a.disableLineBreaks;
        if (noHtml ||
            noHtmlExceptMatchers ||
            disableLineBreaks ||
            markup.match(/<((?:\/[a-z ]+)|(?:[a-z ]+\/))>/gi)) {
            return markup;
        }
        var nextMarkup = markup.replace(/\r\n/g, '\n');
        nextMarkup = nextMarkup.replace(/\n{3,}/g, '\n\n\n');
        nextMarkup = nextMarkup.replace(/\n/g, '<br/>');
        return nextMarkup;
    };
    Parser.prototype.createDocument = function (markup) {
        var doc = document.implementation.createHTMLDocument('Interweave');
        if (INVALID_ROOTS.indexOf(markup.substr(1, ROOT_COMPARE_LENGTH).toUpperCase()) >= 0) {
            if (process.env.NODE_ENV !== 'production') {
                throw new Error('HTML documents as Interweave content are not supported.');
            }
        }
        else {
            doc.body.innerHTML = this.convertLineBreaks(markup);
        }
        return doc;
    };
    Parser.prototype.extractAttributes = function (node) {
        var _this = this;
        var disableWhitelist = this.props.disableWhitelist;
        var attributes = {};
        var count = 0;
        if (node.nodeType !== ELEMENT_NODE || !node.attributes) {
            return null;
        }
        Array.from(node.attributes).forEach(function (attr) {
            var name = attr.name, value = attr.value;
            var newName = name.toLowerCase();
            var filter = constants_1.ATTRIBUTES[newName] || constants_1.ATTRIBUTES[name];
            if (!_this.isSafe(node)) {
                return;
            }
            if (newName.slice(0, ARIA_COMPARE_LENGTH) !== 'aria-') {
                if ((!disableWhitelist && (!filter || filter === constants_1.FILTER_DENY)) ||
                    newName.match(/^on/) ||
                    value.replace(/(\s|\0|&#x0(9|A|D);)/, '').match(/(javascript|vbscript|livescript|xss):/i)) {
                    return;
                }
            }
            var newValue = _this.applyAttributeFilters(newName, value);
            if (filter === constants_1.FILTER_CAST_BOOL) {
                newValue = newValue === 'true' || newValue === newName;
            }
            else if (filter === constants_1.FILTER_CAST_NUMBER) {
                newValue = parseFloat(newValue);
            }
            else {
                newValue = String(newValue);
            }
            attributes[constants_1.ATTRIBUTES_TO_PROPS[newName] || newName] = newValue;
            count += 1;
        });
        if (count === 0) {
            return null;
        }
        return attributes;
    };
    Parser.prototype.getTagConfig = function (tagName) {
        if (constants_1.TAGS[tagName]) {
            return __assign({}, constants_1.TAGS[tagName], { tagName: tagName });
        }
        return {};
    };
    Parser.prototype.isSafe = function (node) {
        if (node instanceof HTMLAnchorElement) {
            var href = node.getAttribute('href');
            if (href && href.charAt(0) === '#') {
                return true;
            }
            var protocol = node.protocol.toLowerCase();
            return (protocol === ':' || protocol === 'http:' || protocol === 'https:' || protocol === 'mailto:');
        }
        return true;
    };
    Parser.prototype.parse = function () {
        return this.parseNode(this.doc.body, __assign({}, constants_1.CONFIG_BLOCK, { tagName: 'body' }));
    };
    Parser.prototype.parseNode = function (parentNode, parentConfig) {
        var _this = this;
        var _a = this.props, noHtml = _a.noHtml, noHtmlExceptMatchers = _a.noHtmlExceptMatchers, disableWhitelist = _a.disableWhitelist, transform = _a.transform;
        var content = [];
        var mergedText = '';
        Array.from(parentNode.childNodes).forEach(function (node) {
            if (node.nodeType === ELEMENT_NODE) {
                var tagName = node.nodeName.toLowerCase();
                var config = _this.getTagConfig(tagName);
                if (constants_1.TAGS_BLACKLIST[tagName]) {
                    return;
                }
                if (mergedText) {
                    content.push(mergedText);
                    mergedText = '';
                }
                var nextNode = _this.applyNodeFilters(tagName, node);
                if (!nextNode) {
                    return;
                }
                var children = void 0;
                if (transform) {
                    _this.keyIndex += 1;
                    var key = _this.keyIndex;
                    children = _this.parseNode(nextNode, config);
                    var transformed = transform(nextNode, children, config);
                    if (transformed === null) {
                        return;
                    }
                    else if (typeof transformed !== 'undefined') {
                        content.push(react_1.default.cloneElement(transformed, { key: key }));
                        return;
                    }
                    _this.keyIndex = key - 1;
                }
                if (!(noHtml || noHtmlExceptMatchers) &&
                    (disableWhitelist || _this.canRenderChild(parentConfig, config))) {
                    _this.keyIndex += 1;
                    var attributes = _this.extractAttributes(nextNode);
                    var elementProps = {
                        tagName: tagName,
                    };
                    if (attributes) {
                        elementProps.attributes = attributes;
                    }
                    if (config.void) {
                        elementProps.selfClose = config.void;
                    }
                    content.push(react_1.default.createElement(Element_1.default, __assign({}, elementProps, { key: _this.keyIndex }), children || _this.parseNode(nextNode, config)));
                }
                else {
                    content = content.concat(_this.parseNode(nextNode, config.tagName ? config : parentConfig));
                }
            }
            else if (node.nodeType === TEXT_NODE) {
                var text = noHtml && !noHtmlExceptMatchers
                    ? node.textContent
                    : _this.applyMatchers(node.textContent || '', parentConfig);
                if (Array.isArray(text)) {
                    content = content.concat(text);
                }
                else {
                    mergedText += text;
                }
            }
        });
        if (mergedText) {
            content.push(mergedText);
        }
        return content;
    };
    return Parser;
}());
exports.default = Parser;
